# DO NOT import any `pytauri` API here,
# Otherwise, in **user** code, if `pytauri` is imported first and then `pytauri_wheel`,
# it will cause a circular import issue:
# `user` -> `pytauri` -> `pytauri_wheel.ext_mod` -> `pytauri.__init__` -> `pytauri` -> `pytauri.__init__` -> ...

"""PyTauri precompiled wheels.

Due to the limitations of circular imports, we cannot import the `pytauri` module in `__init__.py`,
so the related APIs are placed in the [pytauri_wheel.lib][] module.
"""



"""PyTauri precompiled wheels.

# Usage

`pytauri-wheel` provides precompiled [pytauri.EXT_MOD][] for [pytauri][],
which means you can normally use the `pytauri` API, except for the following APIs provided by `pytauri-wheel`:

- [pytauri.builder_factory][] -> [pytauri_wheel.lib.builder_factory][]
- [pytauri.context_factory][] -> [pytauri_wheel.lib.context_factory][]
"""

from pathlib import Path
from typing import Optional

from pytauri import Builder, Context
from pytauri import (
    builder_factory as pytauri_builder_factory,
)
from pytauri import (
    context_factory as pytauri_context_factory,
)

__all__ = ["builder_factory", "context_factory"]


def builder_factory(
    *,
    opener: bool = True,
    clipboard_manager: bool = True,
    dialog: bool = True,
    fs: bool = True,
) -> Builder:
    """A factory function for creating a [pytauri.Builder][] instance.

    This is a type-hinted wrapper function for [pytauri.builder_factory][].

    Args:
        opener: Whether to enable the plugin `opener`
        clipboard_manager: Whether to enable the plugin `clipboard-manager`
        dialog: Whether to enable the plugin `dialog`
        fs: Whether to enable the plugin `fs`
    """
    return pytauri_builder_factory(
        opener=opener, clipboard_manager=clipboard_manager, dialog=dialog, fs=fs
    )


def context_factory(
    src_tauri_dir: Path, /, *, tauri_config: Optional[str] = None
) -> Context:
    """Generate a `Context` based on `tauri.conf.json`, `capabilities/` and etc in the `src_tauri_dir` directory.

    This type-hinted wrapper function for [pytauri.context_factory][].

    Args:
        src_tauri_dir: The **absolute** path of the pytauri project directory.
            In a typical Tauri project, it exists as the `src-tauri` directory;
            in the `pytauri-wheel` project, you only need to provide a similar file layout to `src-tauri`:
                ```text
                src-tauri/
                    __init__.py
                    tauri.conf.json
                    capabilities/
                    ...
                ```
        tauri_config: The config JSON string to be merged with `tauri.conf.json`, equivalent to the `--config` of `tauri-cli`.
            See: <https://tauri.app/develop/configuration-files/#extending-the-configuration>.
    """
    if not src_tauri_dir.is_absolute():
        raise ValueError("`src_tauri_dir` must be an absolute path.")

    return pytauri_context_factory(src_tauri_dir, tauri_config=tauri_config)



"""See: <https://setuptools-rust.readthedocs.io/en/latest/setuppy_tutorial.html>"""

from os import getenv

from packaging.tags import sys_tags
from setuptools import setup
from setuptools_rust import RustExtension

PYTAURI_STANDALONE = getenv("PYTAURI_STANDALONE") == "1"
"""Instead of building pytauri as a extension module file, it will be loaded in memory through Rust's `append_ext_mod`"""

setup(
    rust_extensions=[
        RustExtension(
            # set `target` the same as `[project.entry-points.pytauri.ext_mod]` in `pyproject.toml`
            target="pytauri_wheel.ext_mod",
            # It is recommended to set other features in `Cargo.toml`, except following features:
            features=[
                # see: <https://pyo3.rs/v0.23.3/building-and-distribution.html#the-extension-module-feature>,
                # required to build the extension module
                "pyo3/extension-module",
                # This feature tells Tauri to use embedded frontend assets instead of using a frontend development server.
                # Usually this feature is enabled by `tauri-cli`, here we enable it manually.
                "tauri/custom-protocol",
            ],
        )
    ]
    if not PYTAURI_STANDALONE
    else [],
    # ref:
    # - <https://stackoverflow.com/a/75010995>
    # - <https://github.com/pypa/setuptools/blob/e7c42a0efab982c355667f7cf7ced3bc72f3c7c7/setuptools/command/bdist_wheel.py#L146-L150>
    # - <https://setuptools-rust.readthedocs.io/en/v1.11.0/building_wheels.html#building-for-abi3>
    options={
        "bdist_wheel": {
            # `next(sys_tags()).platform` is usually `manylinux_x_y_{arch}`.
            #
            # `setuptools` usually only assigns `linux_{arch}`, so we need to specify it manually.
            # Although `pytauri-wheel` does not actually comply with the manylinux policy (it link to system webkit lib),
            # we will document these system libs in the README, so it's not a problem.
            "plat_name": next(sys_tags()).platform,
        },
    },
)
